#pragma once

#include <vector>
#include <string>
#include <optional>

#include "BBBParser.h"
#include "sys.h"

class SourceFile {
public:
	enum FileType {
		UNKNOWN, C, CXX,
	};

	static FileType fileTypeFrom(const std::string &name);

	SourceFile(
			std::string dir, std::string name,
			FileType type, BBBParser::Variables vars);

	const std::string &dir() const { return dir_; }
	const std::string &name() const { return name_; }
	FileType type() const { return type_; }
	const BBBParser::Variables &vars() const { return vars_; }

	const std::vector<std::string> *variable(const std::string &name) const;

	const std::vector<std::string> &compileFlags() const;
	const std::vector<std::string> &ldLibs() const;

	std::vector<std::string> dependencies() const;
	bool needsRecompile(const std::string &outDir) const;
	void compile(const std::string &outDir) const;

private:
	std::string dir_;
	std::string name_;
	FileType type_;
	BBBParser::Variables vars_;

	mutable std::vector<std::string> compileFlags_;
	mutable bool hasCompileFlags_ = false;
	mutable std::vector<std::string> ldLibs_;
	mutable bool hasLDLibs_ = false;
};
